from onegov.winterthur.collections.address import AddressCollection
from onegov.winterthur.collections.address import AddressSubsetCollection

__all__ = ('AddressCollection', 'AddressSubsetCollection')
